# HIVE AdKit C++ SDK

HIVE AdKit C++ SDK는 ADOP 제품을 Cocos2dx C++ 인터페이스로 제공합니다.

HIVE AdKit C++ SDK는 HIVE SDK와 함께 사용하는 경우 RewardVideo시청 보상이 도달하였을때, HIVE Analytics에 시청 완료 정보를 전송합니다.
HIVE SDK가 포함되지 않거나, HIVE SDK가 초기화가 되지 않은 경우에는 동작하지 않습니다.

HIVE AdKit C++ SDK는 *iOS*와 *Andriod*에서 동작하며, 각 플랫폼마다 필요한 설정요소의 차이가 있습니다.

## *iOS* 에서 사용

* Xcode 12 이상에서 빌드하는것을 추천합니다.

1. 최소지원 버전은 iOS 11 입니다.

1. HIVEAdKit/src와 HIVEAdKit/include를 XCode프로젝트에 드레그&드랍의 동작으로 포함시킴니다.

   * .cpp .hpp .mm 파일을 모두 포함합니다.

   * HIVE AdKit 코드는 Brdige코드로 실제 사용성을 위해 제공합니다.

1. CocoaPods의 설정으로 아래의 내용을 추가합니다.

    ``` pod
    platform :ios, '11.0'

    target 'MyGame-mobile' do
        use_frameworks! :linkage => :static

        pod 'BidmadSDK', '4.0.1.1'
        pod 'OpenBiddingHelper', '4.0.1.1'
        pod 'BidmadAdapterFNC/ForGame', '4.0.0.2'
        pod 'BidmadAdapterFC', '4.0.0.0'
    end
    ```

1. info.plist에 GADApplicationIdentifier를 추가합니다. *GADApplicationIdentifier는 Google Admob에서 확인할 수 있습니다.

    ``` plist
    <key>GADApplicationIdentifier</key>
    <string>ca-app-pub-XXXXXX~XXXXXX</string>
    ```

1. iOS 14 이상을 대상으로 하는 경우 다음의 내용을 info.plist 파일에 추가합니다.

    * ADOP의 SKAdNetwork 목록 : https://github.com/bidmad/Bidmad-Cocos2dx/wiki/Preparing-for-iOS-14%5BKOR%5D

## *Android* 에서 사용

* Android Studio 2020.3 이상에서 빌드하는것을 추천합니다.
* Android에서 사용하는 경우, Prebuilt-Firebase-Library와 포함된 HIVE AdKit SourceCode를 빌드하여 사용할 수 있습니다.

1. 최소지원 버전은 Android SDK 19 입니다.
 
1. ExternalNativeBuild
    * Android.mk 는 더이상 지원하지 않습니다. (Android 지원종료)

    * cmake

    'build.gradle'

    ``` gradle
    externalNativeBuild {
        cmake {
            targets 'MyGame'
            arguments "-DCMAKE_FIND_ROOT_PATH=", "-DANDROID_STL=c++_static", "-DANDROID_TOOLCHAIN=clang", "-DANDROID_ARM_NEON=TRUE"
            cppFlags "-frtti -fexceptions -fsigned-char"
            path "jni/CMakeLists.txt"
        }
    }
    ndk {
            abiFilters 'x86', 'armeabi-v7a'//, 'arm64-v8a' // 'armeabi'는 더이상 지원하지 않음
        }
    ```

    'CMakeList.txt'

    ``` cmake
    set(HIVEADKIT_DIR_PATH ....../HIVEAdKit) #setup to HIVEAdKit path
    #HIVEAdKit library import after add_library( MyGame ... )
    add_subdirectory(${HIVEADKIT_DIR_PATH}) #HIVEAdKit library & macro load
    use_hiveadkit_prebuilt(${HIVEADKIT_DIR_PATH} MyGame) #library import automatic macro to build target
    ```

1. build.gradle 파일에 다음의 내용을 추가합니다.

    ``` gradle

    apply from: '......../HIVEAdkit/adkit.gradle'

    android {
      ...
      sourceSets.main {
          java.srcDirs "src" , "......./HIVEAdKit/Android/java" // HIVEAdKit JavaPlugin Source Path
      }
      ...
    }

    ```

1. AndroidManifest.xml 파일에 다음의 내용을 추가합니다.[1]

    ``` AndroidManifest.xml
    <application>
      <meta-data android:name="com.google.android.gms.ads.AD_MANAGER_APP" android:value="true" />
      <meta-data
        android:name="com.google.android.gms.ads.APPLICATION_ID"
        android:value="ca-app-pub-3940256099942544~3347511713"/> <!-- Sample value -->
    </application>
    ```

[1] https://developers.google.com/admob/android/quick-start#update_your_androidmanifestxml
