//
//  AdMobTest.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by nanomech on 18/05/2020.
//
#include "HIVEAdKit.hpp"
#include "AdKitTest.hpp"

#if defined(ANDROID) // Android NDK predefined
#include <jni.h>
#include <HIVE_SDK_Plugin/HIVE_JniHelper.h>
// #include "HIVE_JniHelper.h"
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
#include "platform/android/jni/JniHelper.h"
#endif
#endif


#if defined(ANDROID)
JNIEnv* GetJniEnv() {
    // TODO : GetJniEnv() for exclusive Cocos2D Engine.
    // TODO : GetJniEnv() implements for each Engine.
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
    return cocos2d::JniHelper::getEnv();
#elif defined(ANDROID) // Android NDK predefined
    return hive::HiveJniHelper::getEnv();
#else
    return NULL;
#endif
}

jobject GetActivity() {
    // TODO : GetActivity() for exclusive Cocos2D Engine.
    // TODO : GetActivity() implements for each Engine.
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID) // Cocos2D predefined
    return cocos2d::JniHelper::getActivity();
#elif defined(ANDROID) // Android NDK predefined
//    return hive::HiveJniHelper::getActivity(); // Not Supported
    return NULL;
#else
    return NULL
#endif
}
#endif

#if defined(ANDROID)
const char* rewardVideoUnitId =      "7d9a2c9e-5755-4022-85f1-6d4fc79e4418";
const char* InterstitialAdUnitId =   "e9acd7fc-a962-40e4-aaad-9feab1b4f821";
const char* AdaptiveBannerAdUnitId = "944fe870-fa3a-4d1b-9cc2-38e50b2aed43";
    
#else// UNITY_IOS
const char* rewardVideoUnitId =      "29e1ef67-98d2-47b3-9fa2-9192327dd75d";
const char* InterstitialAdUnitId =   "228b95a9-6f42-46d8-a40d-60f17f751eb1";
const char* AdaptiveBannerAdUnitId = "1c3e3085-333f-45af-8427-2810c26a72fc";
#endif

using namespace hive;

bool AdKitTest::init()
{
    if (!BaseView::init())
        return false;

    setUseBackButton(true);
    setUseShowLogButton(true);

    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage) {
        addLogMessage(logMessage);
    });
    addLabel(":: HIVE AdKit Test ::");
    
    addDottedLine();
    addButton("GDPR_RESET", BUTTON_GDPR_RESET);
#if defined(ANDROID)
    addButton("GDPR_INIT_NO_TEST", BUTTON_GDPR_INIT0);
    addButton("픽셀4a(OCB218)", BUTTON_GDPR_INIT1);
    addButton("갤럭시S10(AD-29)", BUTTON_GDPR_INIT2);
    addButton("GDPR_INIT_NO_TEST", BUTTON_GDPR_INIT3);
#else// UNITY_IOS
    addButton("GDPR_INIT_NO_TEST", BUTTON_GDPR_INIT0);
    addButton("김재원", BUTTON_GDPR_INIT1);
    addButton("아이폰13프로(IOSP-23)", BUTTON_GDPR_INIT2);
    addButton("아이폰12프로맥스(IOSP-18)", BUTTON_GDPR_INIT3);
#endif
    addButton("INIT_ADKIT", BUTTON_INIT_ADKIT);
    addButton("INIT_REWARDED_VIDEO", BUTTON_INIT_REWARDED_VIDEO);
    addButton("LOAD_REWARDED_VIDEO", BUTTON_LOAD_REWARDED_VIDEO);
    addButton("SHOW_REWARDED_VIDEO", BUTTON_SHOW_REWARDED_VIDEO);
    addDottedLine();
    
    addButton("INIT_INTERSTITIAL", BUTTON_INIT_INTERSTITIAL_AD);
    addButton("LOAD_INTERSTITIAL", BUTTON_LOAD_INTERSTITIAL_AD);
    addButton("SHOW_INTERSTITIAL", BUTTON_SHOW_INTERSTITIAL_AD);
    addDottedLine();
    
    addButton("INIT_ADAPTIVE_BANNER", BUTTON_INIT_ADAPTIVE_BANNER);
    addButton("LOAD_ADAPTIVE_BANNER", BUTTON_LOAD_ADAPTIVE_BANNER);
    addButton("SHOW_ADAPTIVE_BANNER", BUTTON_SHOW_ADAPTIVE_BANNER);
    addButton("HIDE_ADAPTIVE_BANNER", BUTTON_HIDE_ADAPTIVE_BANNER);
    addDottedLine();

    setUIEnabled(true, BUTTON_GDPR_RESET);
    setUIEnabled(true, BUTTON_GDPR_INIT0);
    setUIEnabled(false, BUTTON_GDPR_INIT1);
    setUIEnabled(false, BUTTON_GDPR_INIT2);
    setUIEnabled(false, BUTTON_GDPR_INIT3);
    
    setUIEnabled(false, BUTTON_INIT_REWARDED_VIDEO);
    setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
    setUIEnabled(false, BUTTON_SHOW_REWARDED_VIDEO);
    rewardVideo = NULL;
    
    setUIEnabled(false, BUTTON_INIT_INTERSTITIAL_AD);
    setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
    setUIEnabled(false, BUTTON_SHOW_INTERSTITIAL_AD);
    interstitial = NULL;
    
    setUIEnabled(false, BUTTON_INIT_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_LOAD_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_SHOW_ADAPTIVE_BANNER);
    setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
    adaptiveBanner = NULL;

    return true;
}

void AdKitTest::onButtonEvent(int buttonTag)
{   
    /**
     BUTTON_INIT_ADKIT,
     BUTTON_INIT_REWARDED_VIDEO,
     BUTTON_LOAD_REWARDED_VIDEO,
     BUTTON_SHOW_REWARDED_VIDEO,
     BUTTON_INIT_INTERSTITIAL_AD,
     BUTTON_LOAD_INTERSTITIAL_AD,
     BUTTON_SHOW_INTERSTITIAL_AD,
     BUTTON_INIT_ADAPTIVE_BANNER,
     BUTTON_LOAD_ADAPTIVE_BANNER,
     BUTTON_SHOW_ADAPTIVE_BANNER,
     BUTTON_HIDE_ADAPTIVE_BANNER,
     */
    switch (buttonTag)
    {
        case BUTTON_GDPR_RESET:
            setUIEnabled(false, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(false, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(false, BUTTON_INIT_ADAPTIVE_BANNER);
            HIVEAdKit::GDPR::Reset();
            setUIEnabled(true, BUTTON_GDPR_INIT1);
            setUIEnabled(true, BUTTON_GDPR_INIT2);
#if !defined(ANDROID)
            setUIEnabled(true, BUTTON_GDPR_INIT3);
#endif
            break;
        case BUTTON_GDPR_INIT0:
            setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
            HIVEAdKit::GDPR::Setup(false, nullptr);
            break;
        case BUTTON_GDPR_INIT1:
            setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
#if defined(ANDROID)
            HIVEAdKit::GDPR::Setup(true, "8BC53EF9F2715181203EA56580344C07");
#else// UNITY_IOS
            HIVEAdKit::GDPR::Setup(true, "C78E6400-6007-4E7F-8B37-6FE99157B4C8");
#endif
            break;
        case BUTTON_GDPR_INIT2:
            setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
#if defined(ANDROID)
            HIVEAdKit::GDPR::Setup(true, "44F856EACB2C62E02E8BD9EFF546BF09");
#else// UNITY_IOS
            HIVEAdKit::GDPR::Setup(true, "E52BAA6D-4C31-4342-9D57-CE41B6BA5B90");
#endif
            break;
        case BUTTON_GDPR_INIT3:
            setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
#if defined(ANDROID)
            HIVEAdKit::GDPR::Setup(false, nullptr);
#else// UNITY_IOS
            HIVEAdKit::GDPR::Setup(true, "1B2D1BB4-DFF0-40AF-ABD5-7FCE4F560818");
#endif
            break;
            /// RewardedVideo
        case BUTTON_INIT_REWARDED_VIDEO:
            rewardVideo = HIVEAdKit::RewardVideo::Initialize(rewardVideoUnitId, this);
            setUIEnabled(false, BUTTON_INIT_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_LOAD_REWARDED_VIDEO);
            break;
        case BUTTON_LOAD_REWARDED_VIDEO:
            //Load
            HIVEAdKit::RewardVideo::Load(*rewardVideo, "CocosCPP-RewardVideo-Load-AdPlacementInfo");
            setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
            break;
        case BUTTON_SHOW_REWARDED_VIDEO:
            //Show
            HIVEAdKit::RewardVideo::Show(*rewardVideo, "CocosCPP-RewardVideo-Show-AdPlacementInfo");
            setUIEnabled(false, BUTTON_SHOW_REWARDED_VIDEO);
            break;

            /// Interstitial
        case BUTTON_INIT_INTERSTITIAL_AD:
            if( interstitial != nullptr ) HIVEAdKit::Interstitial::Destroy(*interstitial);
            interstitial = HIVEAdKit::Interstitial::Initialize(InterstitialAdUnitId, this);
            setUIEnabled(false, BUTTON_INIT_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_LOAD_INTERSTITIAL_AD);
            break;
        case BUTTON_LOAD_INTERSTITIAL_AD:
            //Load
            HIVEAdKit::Interstitial::Load(*interstitial, "CocosCPP-Interstitial-Load-AdPlacementInfo");
            setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
            break;
        case BUTTON_SHOW_INTERSTITIAL_AD:
            //Show
            HIVEAdKit::Interstitial::Show(*interstitial, "CocosCPP-Interstitial-Show-AdPlacementInfo");
            setUIEnabled(false, BUTTON_SHOW_INTERSTITIAL_AD);
            break;

            /// Adaptive Banner
        case BUTTON_INIT_ADAPTIVE_BANNER:
            // Standard mobile banner size is 320x50.
            static const int kBannerWidth = 320;
            static const int kBannerHeight = 50;
            adaptiveBanner = HIVEAdKit::AdaptiveBanner::Initialize(AdaptiveBannerAdUnitId, this, kBannerHeight);
            setUIEnabled(false, BUTTON_INIT_ADAPTIVE_BANNER);
            setUIEnabled(true, BUTTON_LOAD_ADAPTIVE_BANNER);
            break;
        case BUTTON_LOAD_ADAPTIVE_BANNER:
            //Load
            HIVEAdKit::AdaptiveBanner::Load(*adaptiveBanner, "CocosCPP-AdaptiveBanner-Load-AdPlacementInfo");
            setUIEnabled(false, BUTTON_LOAD_ADAPTIVE_BANNER);
            setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
            break;
        case BUTTON_SHOW_ADAPTIVE_BANNER:
            //Show
            HIVEAdKit::AdaptiveBanner::Show(*adaptiveBanner, "CocosCPP-AdaptiveBanner-Show-AdPlacementInfo");
            setUIEnabled(false, BUTTON_SHOW_ADAPTIVE_BANNER);
            setUIEnabled(true, BUTTON_HIDE_ADAPTIVE_BANNER);
            break;
        case BUTTON_HIDE_ADAPTIVE_BANNER:
            //Hide
            HIVEAdKit::AdaptiveBanner::Hide(*adaptiveBanner);
            setUIEnabled(true, BUTTON_SHOW_ADAPTIVE_BANNER);
            setUIEnabled(false, BUTTON_HIDE_ADAPTIVE_BANNER);
            break;
    }
}

void AdKitTest::onAdLoaded(const char* zoneId) {
    Logger::log("onAdLoaded\n");
    if (strcmp(zoneId,rewardVideoUnitId) == 0) {
        setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
        setUIEnabled(true, BUTTON_SHOW_REWARDED_VIDEO);
    } else if (strcmp(zoneId,InterstitialAdUnitId) == 0) {
        setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
        setUIEnabled(true, BUTTON_SHOW_INTERSTITIAL_AD);
    } else if (strcmp(zoneId,AdaptiveBannerAdUnitId) == 0) {
        setUIEnabled(false, BUTTON_LOAD_ADAPTIVE_BANNER);
        setUIEnabled(true, BUTTON_SHOW_ADAPTIVE_BANNER);
    }
    
}
void AdKitTest::onAdOpening(const char* zoneId) {
    Logger::log("onAdOpening\n");
}
void AdKitTest::onAdClosed(const char* zoneId) {
    Logger::log("onAdClosed\n");
    if (strcmp(zoneId,rewardVideoUnitId) == 0) {
        if( HIVEAdKit::RewardVideo::IsLoaded(*rewardVideo) ) {
            setUIEnabled(false, BUTTON_LOAD_REWARDED_VIDEO);
            setUIEnabled(true, BUTTON_SHOW_REWARDED_VIDEO);
        } else if( HIVEAdKit::RewardVideo::IsInitialized(*rewardVideo) ) {
            setUIEnabled(true, BUTTON_LOAD_REWARDED_VIDEO);
        }
    } else if (strcmp(zoneId,InterstitialAdUnitId) == 0) {
        if( HIVEAdKit::Interstitial::IsLoaded(*interstitial) ) {
            setUIEnabled(false, BUTTON_LOAD_INTERSTITIAL_AD);
            setUIEnabled(true, BUTTON_SHOW_INTERSTITIAL_AD);
        } else if( HIVEAdKit::Interstitial::IsInitialized(*interstitial) ) {
            setUIEnabled(true, BUTTON_LOAD_INTERSTITIAL_AD);
        }
    } else if (strcmp(zoneId,AdaptiveBannerAdUnitId) == 0) {
    }
}
void AdKitTest::onAdReward(const char* zoneId) {
    Logger::log("onAdReward\n");
}
void AdKitTest::onAdClick(const char* zoneId) {
    Logger::log("onAdClick\n");
}
void AdKitTest::onAdSkip(const char* zoneId) {
    Logger::log("onAdSkip\n");
}
void AdKitTest::onAdFail(const char* zoneId) {
    Logger::log("onAdFail\n");
    if (strcmp(zoneId,rewardVideoUnitId) == 0) {
        setUIEnabled(true, BUTTON_INIT_REWARDED_VIDEO);
    } else if (strcmp(zoneId,InterstitialAdUnitId) == 0) {
        setUIEnabled(true, BUTTON_INIT_INTERSTITIAL_AD);
    } else if (strcmp(zoneId,AdaptiveBannerAdUnitId) == 0) {
        setUIEnabled(true, BUTTON_INIT_ADAPTIVE_BANNER);
    }
}
