//
//  HIVEAdKit.hpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by nanomech on 06/04/2020.
//  Modifyed by nanomech on 11/18/2021.
//  Customized for ADOP-Bidmad
//

#ifndef HIVEAdKit_hpp
#define HIVEAdKit_hpp

#include <stdio.h>
#include <string>

#include "bidmad/InterstitialInterface.h"
#include "bidmad/BannerInterface.h"
#include "bidmad/RewardInterface.h"
#include "bidmad/GoogleGDPRInterface.h"
#include "bidmad/CommonInterface.h"

//#if TARGET_IPHONE_SIMULATOR
//extern const char* _getSimulatorDevice(void);
//#endif
extern "C" {
    void HIVEAnlayticsLog_AdLog(const char* eventType, const char* adUnitId, const char* uniqueSessionKey, const char* network, const char* adPlacementInfo, const char* rewardType, double rewardAmount);
    const char* HIVEAnalyticsLog_CreateUniqueSessionKey();
} // extern "C"

#if defined(__ANDROID__)
// Get the JNI environment.
JNIEnv* GetJniEnv();
// Get Android Activity
jobject GetActivity();
#endif  // defined(__ANDROID__)

// extern firebase::admob::AdParent getAdMobViewParent();

class HIVEAdKit
{

private:
    const std::string TAG = "HIVEAdKit::";

public:
    class EventHandlers {
    public:
        virtual void onAdLoaded(const char*) = 0;
        virtual void onAdOpening(const char*) = 0;
        virtual void onAdClosed(const char*) = 0;
        virtual void onAdSkip(const char*) = 0;
        virtual void onAdReward(const char*) = 0;
        virtual void onAdClick(const char*) = 0;
        virtual void onAdFail(const char*) = 0;
    };
    static void AdMobInitialize()
    {
        // Debug.Log(TAG + "AdMobInitialize");
        getInstance().bInit = true;
    }
private:
    bool bInit;  
    HIVEAdKit() {bInit = false;}
    ~HIVEAdKit() {}
    
    static HIVEAdKit& getInstance() {
        static HIVEAdKit* instance = nullptr;
        if( instance == nullptr )
            instance = new HIVEAdKit();
        return *instance;
    }
public:
    class GDPR
    {
    private:
        GoogleGDPRInterface gdprGoogle;// = nullptr;

    // requestConsentInfoUpdate() callback -> success 동의 요청이 완료된 이후 들어오는 콜백.
    static void ConsentInfoUpdateSuccessCallback() {
        // Debug.Log("ConsentInfoUpdateSuccessCallback");
        
        // 2. 동의 팝업 로드 가능 여부를 확인합니다.
        if (getInstance().gdprGoogle.isConsentFormAvailable()) {
            // 3. 동의 팝업을 로드합니다.
            getInstance().gdprGoogle.loadForm();
        } else {
            AdMobInitialize();
        }
    }
    // requestConsentInfoUpdate() callback ->failed
    static void ConsentInfoUpdateFailureCallback(char* errorLog) {
        // 동의 팝업 요청 실패
        // Debug.Log("ConsentInfoUpdateFailureCallback: " + errorLog);
        AdMobInitialize();
    }

    // loadForm() callback -> success 1. 동의 팝업 로드가 완료된 이후 들어오는 콜백.
    static void ConsentFormLoadSuccessCallback() {
        // Debug.Log("ConsentFormLoadSuccessCallback");
        // 2. 동의가 필요한지에 대한 여부를 확인합니다 (동의가 필요한 상태)
        int state = getInstance().gdprGoogle.getConsentStatus();
        
        if (state == 1) { // 동의 요청이 들어오지 않은 상태입니다.
            getInstance().gdprGoogle.showForm();// 3. 동의가 필요하기 때문에 동의 팝업을 디스플레이 합니다.
        } else if (state == 0) {
            // Debug.Log("ConsentStatusUnknown");
        } else if (state == 2) {// GDPR 동의가 필요 없는 상태입니다.
            // Debug.Log("ConsentStatusNotRequired");
            AdMobInitialize();
        } else if (state == 3) {// GDPR 동의를 이미 받은 상태입니다.
            // Debug.Log("ConsentStatusObtained");
            AdMobInitialize();
        }
    }

    // loadForm() callback -> fail 동의 팝업 폼 로드 실패
    static void ConsentFormLoadFailureCallback(char* errorLog) {
        // Debug.Log("ConsentFormLoadFailureCallback: " + errorLog);
        AdMobInitialize();
    }

    // 사용자와의 상호작용이 끝나고 결과를 보내는 콜백. formError가 null 일 경우, 동의 혹은 미동의 수신을 성공했음을 뜻합니다. 
    static void ConsentFormDismissedCallback(char* errorLog) {
        // Debug.Log("ConsentFormDismissedCallback: " + errorLog);
        int state = getInstance().gdprGoogle.getConsentStatus();
        if( state == 1 ) { // 동의 요청이 들어오지 않은 상태입니다.
            getInstance().gdprGoogle.showForm(); // 3. 동의가 필요하기 때문에 동의 팝업을 디스플레이 합니다.
        } else {
            AdMobInitialize();
        }
    }
    protected:
        GDPR() {
//            gdprGoogle = new GoogleGDPRInterface();
            gdprGoogle.setOnConsentInfoUpdateSuccessCallback(&GDPR::ConsentInfoUpdateSuccessCallback);
            gdprGoogle.setOnConsentInfoUpdateFailureCallback(&GDPR::ConsentInfoUpdateFailureCallback);
            gdprGoogle.setOnConsentFormLoadSuccessCallback(&GDPR::ConsentFormLoadSuccessCallback);
            gdprGoogle.setOnConsentFormLoadFailureCallback(&GDPR::ConsentFormLoadFailureCallback);
            gdprGoogle.setOnConsentFormDismissedCallback(&GDPR::ConsentFormDismissedCallback);
        }
        
        static GDPR& getInstance() {
            static GDPR* instance = nullptr;
            if( instance == nullptr )
                instance = new GDPR();
            return *instance;
        }
    public:
        static void Setup(bool debugState, char* testDeviceId) {
            if( debugState && testDeviceId != nullptr ) {
                    getInstance().gdprGoogle.setDebug(testDeviceId, true);
            }
            getInstance().gdprGoogle.requestConsentInfoUpdate();
        }
        static void Reset() {
            getInstance().gdprGoogle.reset();
        }
    };

    class AdLog
    {
    private:
//        static std::string adNetwork = "admob"; /// Fixed
        std::string adSessionKey;
        EventHandlers* callback;

    protected:
        std::string adUnitId;
        std::string adPlacementInfo;
        
    protected:
        AdLog(std::string unitId, EventHandlers* pEventHanders) {
            adUnitId = unitId;
            callback = pEventHanders;
            adSessionKey = std::string(HIVEAnalyticsLog_CreateUniqueSessionKey());
            getInstanace(unitId, this);
        }
        AdLog(const AdLog&) {
            
        }
        virtual ~ AdLog() {
            
        }
    public:
        
        static AdLog* getInstanace(std::string unitId, AdLog* value = nullptr) {
            static std::map<std::string, AdLog*> zoneMap = std::map<std::string, AdLog*>();
            if( value != nullptr ) zoneMap[unitId] = value;
            auto it = zoneMap.find(unitId);
            if( it != zoneMap.end() ) return it->second;
            return nullptr;
        }
        
    public:
        
        // Common Event
        void AdLoaded() {
            // load
            HIVEAnlayticsLog_AdLog("load", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdLoaded(adUnitId.c_str());
        }
        
        void AdOpening() {
            // open
            HIVEAnlayticsLog_AdLog("open", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdOpening(adUnitId.c_str());
        }
        
        void AdClosed() {
            // closed
            HIVEAnlayticsLog_AdLog("closed", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdClosed(adUnitId.c_str());
        }
        
        // RewardedAd Event
        void AdReward() {
            // reward
            HIVEAnlayticsLog_AdLog("reward", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdReward(adUnitId.c_str());
        }
        
        // InterstitialAd, AdaptiveBanner Event
        void AdLeavingApplication() {
            // click
            HIVEAnlayticsLog_AdLog("click", adUnitId.c_str(), adSessionKey.c_str(), "admob", adPlacementInfo.c_str(), NULL, 0.0f);
            if( callback != NULL ) callback->onAdClick(adUnitId.c_str());
        }

        void AdFailed() {
            // failed
            if( callback != NULL ) callback->onAdFail(adUnitId.c_str());
        }
        
        void AdSkip() {
            // skip ad
            if( callback != NULL ) callback->onAdSkip(adUnitId.c_str());
        }
        
    };
    class Interstitial : public AdLog
    {
    private:
        InterstitialInterface* interstitial_ad;
        
        Interstitial(std::string unitId, EventHandlers* pEventHanders) : AdLog(unitId, pEventHanders) {
            //Interstitial Create

            interstitial_ad = new InterstitialInterface((char*)(adUnitId.c_str()));
            interstitial_ad->setAutoReload(false);
            
            //Callback Setting
            interstitial_ad->setOnLoadCallback(&Interstitial::onInterstitialLoad);
            interstitial_ad->setOnShowCallback(&Interstitial::onInterstitialShow);
            interstitial_ad->setOnFailCallback(&Interstitial::onInterstitialFail);
            interstitial_ad->setOnCloseCallback(&Interstitial::onInterstitialClose);
        }
        ~Interstitial() {
            if( interstitial_ad != nullptr ) {
                delete interstitial_ad;
            }
        }

        static void onInterstitialLoad(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdLoaded();
        }
        static void onInterstitialShow(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdOpening();
        }
        static void onInterstitialClose(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdClosed();
        }
        static void onInterstitialFail(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdFailed();
        }
    public:
        Interstitial& operator=(const Interstitial &rhs) {
            return *this;  // 자기 자신을 참조를 반환합니다.
        }
        
        static Interstitial* Initialize(std::string unitId, EventHandlers* pEventHanders)
        {
            return (new Interstitial(unitId, pEventHanders));
        }
        static bool IsInitialized(Interstitial& ad) {
            return ad.interstitial_ad != nullptr;
        }
        static bool IsLoaded(Interstitial& ad) {
            return ad.interstitial_ad != nullptr && ad.interstitial_ad->isLoaded();
        }
        static void Load(Interstitial& ad, std::string adPlacementInfo) {
            if (IsInitialized(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.interstitial_ad->load();
            }
        }
        static void Show(Interstitial& ad, std::string adPlacementInfo) {
            if( IsLoaded(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.interstitial_ad->show();
            }
        }
        static void Destroy(Interstitial& ad) {
            if( IsInitialized(ad)) {
                delete &ad;
            }
        }
    };
    
    class AdaptiveBanner : public AdLog
    {
    private:
        BannerInterface* banner_view;
        
        int top;
        AdaptiveBanner(const char* unitId, EventHandlers* pEventHanders, int y) : AdLog(unitId, pEventHanders) {
            //Banner Create
            banner_view = new BannerInterface((char*)(adUnitId.c_str()));

            // bi->setInterval(60);
            top = y;

            //Callback Setting
            banner_view->setOnLoadCallback(&AdaptiveBanner::onBannerLoad);
            banner_view->setOnFailCallback(&AdaptiveBanner::onBannerFail);
        }
        ~AdaptiveBanner() {
            if( banner_view != nullptr) {
                banner_view->removeBanner();
                delete banner_view;
            }
        }
        static void onBannerLoad(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdLoaded();
            dynamic_cast<AdaptiveBanner*>(AdLog::getInstanace(zoneId))->bLoaded = true;
        }
        static void onBannerFail(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdFailed();
        }

    public:
        bool bLoaded = false;
        AdaptiveBanner& operator=(const AdaptiveBanner &rhs) {
            return *this;  // 자기 자신을 참조를 반환합니다.
        }
        
        static AdaptiveBanner* Initialize(const char* unitId, EventHandlers* pEventHanders, int y)
        {
            return (new AdaptiveBanner(unitId, pEventHanders, y));
        }
        
        static bool IsInitialize(AdaptiveBanner& ad) {
            return ad.banner_view != nullptr;
        }
        static bool IsLoaded(AdaptiveBanner& ad) {
            return ad.banner_view != nullptr && ad.bLoaded;
        }
        static void Load(AdaptiveBanner& ad, std::string adPlacementInfo) {
            if (IsInitialize(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.banner_view->load(ad.top);
            }
        }
        static void Show(AdaptiveBanner& ad, std::string adPlacementInfo) {
            if( IsLoaded(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.banner_view->showBannerView();
                ad.AdOpening();
            }
        }
        static void Hide(AdaptiveBanner& ad) {
            if( IsLoaded(ad)) {
                ad.banner_view->hideBannerView();
                ad.AdClosed();
            }
        }
        static void Destory(AdaptiveBanner& ad) {
            if( IsInitialize(ad)) {
                delete &ad;
            }
        }
    };
    
    class RewardVideo : public AdLog
    {
    protected:
        RewardInterface* pRV;
        RewardVideo(std::string unitId, EventHandlers* pEventHanders) : AdLog(unitId, pEventHanders) {
            //Reward Create
            pRV = new RewardInterface((char*)(adUnitId.c_str()));
            pRV->setAutoReload(false);
            //Callback Setting
            pRV->setOnLoadCallback(&RewardVideo::onRewardLoad);
            pRV->setOnShowCallback(&RewardVideo::onRewardShow);
            pRV->setOnCompleteCallback(&RewardVideo::onRewardComplete);
            pRV->setOnCloseCallback(&RewardVideo::onRewardClose);
            pRV->setOnFailCallback(&RewardVideo::onRewardFail);
            pRV->setOnSkipCallback(&RewardVideo::onRewardSkip);
        }
        ~RewardVideo() {
            if( pRV != nullptr )
                delete pRV;
        }

        static void onRewardLoad(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdLoaded();
        }
        static void onRewardShow(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdOpening();
        }
        static void onRewardComplete(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdReward();
        }
        static void onRewardSkip(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdSkip();
        }
        static void onRewardClose(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdClosed();
        }
        static void onRewardFail(char* zoneId)
        {
            AdLog::getInstanace(zoneId)->AdFailed();
        }
    public:
        static RewardVideo* Initialize(std::string unitId, EventHandlers* pEventHanders)
        {
            return (new RewardVideo(unitId, pEventHanders));
        }

        static bool IsInitialized(RewardVideo& ad) {
            return ad.pRV != nullptr;
        }

        static bool IsLoaded(RewardVideo& ad) {
            return (ad.pRV != nullptr) && (ad.pRV->isLoaded());
        }
        
        static void Load(RewardVideo& ad, std::string adPlacementInfo)
        {
            if (IsInitialized(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.pRV->load();
            }
        }
        
        static void Show(RewardVideo &ad, std::string adPlacementInfo)
        {
            if( IsLoaded(ad)) {
                ad.adPlacementInfo = adPlacementInfo;
                ad.pRV->show();
            }
        }
    };
};

#endif /* HIVEAdKit_hpp */
