//
//  HIVEAdKit.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by nanomech on 06/04/2020.
//

#include "HIVEAdKit.hpp"

#include <string.h>
#include <string>

#if defined(ANDROID) || defined(__ANDROID__)
#include <jni.h>
#include <android/log.h>
#include <pthread.h>

const char* HIVE_ANALYTICS_PLUGIN_CLASS = "com/com2us/module/HIVEAnalyticsLogPlugin";

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
//#include "platform/android/jni/JniHelper.h"
#endif
#include <unordered_map>

typedef std::unordered_map<JNIEnv*, std::vector<jobject>> LocalRefMapType;

static std::string getStringUTFCharsJNI(JNIEnv* env, jstring srcjStr)
{
    std::string ret;
    if(srcjStr != nullptr && env != nullptr)
    {
        const char* utf8Str = (const char*)env->GetStringUTFChars(srcjStr, nullptr);
        ret = utf8Str;
        env->ReleaseStringUTFChars(srcjStr, utf8Str);
    }
    return ret;
}

static jstring newStringUTFJNI(JNIEnv* env, const std::string& utf8Str)
{
    jstring stringText = env->NewStringUTF(utf8Str.c_str());
    return stringText;
}

static jstring convert(LocalRefMapType& localRefs, JNIEnv* env, const char* x) {
    jstring ret = newStringUTFJNI(env, x ? x : "");
    localRefs[env].push_back(ret);
    return ret;
}

static jstring convert(LocalRefMapType& localRefs, JNIEnv* env, const std::string& x) {
    return convert(localRefs, env, x.c_str());
}

template <typename T>
static T convert(LocalRefMapType& localRefs, JNIEnv*, T x) {
    return x;
}

static void deleteLocalRefs(JNIEnv* env, LocalRefMapType& localRefs) {
    if (!env) {
        return;
    }

    for (const auto& ref : localRefs[env]) {
        env->DeleteLocalRef(ref);
    }
    localRefs[env].clear();
}

static std::string getJNISignature() {
    return "";
}

static  std::string getJNISignature(double) {
    return "D";
}

static std::string getJNISignature(const char*) {
    return "Ljava/lang/String;";
}

static std::string getJNISignature(const std::string&) {
    return "Ljava/lang/String;";
}

template <typename T>
static std::string getJNISignature(T x) {
    // This template should never be instantiated
    static_assert(sizeof(x) == 0, "Unsupported argument type");
    return "";
}

template <typename T, typename... Ts>
static std::string getJNISignature(T x, Ts... xs) {
    return getJNISignature(x) + getJNISignature(xs...);
}

template <typename... Ts>
void HIVEAnalyticsLogPlugin_callJNI(const std::string methodName, Ts... arguments)
{
//#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
//    cocos2d::JniHelper::callStaticVoidMethod(HIVE_ANALYTICS_PLUGIN_CLASS, methodName, arguments...);
//#else
    jclass cls;
    jmethodID methodId;

    JNIEnv *env = GetJniEnv();
    if(env == NULL)
    {
        return;
    }

    cls = env->FindClass(HIVE_ANALYTICS_PLUGIN_CLASS);
    if (env->ExceptionCheck())
    {
        jthrowable exception = env->ExceptionOccurred();
        env->ExceptionDescribe();
        env->ExceptionClear();
        env->DeleteLocalRef(exception);
        return;
    }
    std::string signature = "(" + std::string(getJNISignature(arguments...)) + ")V";

    methodId = env->GetStaticMethodID(cls, methodName.c_str(), signature.c_str());
    if (env->ExceptionCheck())
    {
        jthrowable exception = env->ExceptionOccurred();
        env->ExceptionClear();
        env->DeleteLocalRef(exception);
        return;
    }
    LocalRefMapType localRefs;
    env->CallStaticVoidMethod(cls, methodId, convert(localRefs, env, arguments)...);
    env->DeleteLocalRef(cls);
    deleteLocalRefs(env, localRefs);
//#endif
}

template <typename... Ts>
std::string HIVEAnalyticsLogPlugin_callJNI_backString(const std::string methodName, Ts... arguments)
{
    jclass cls;
    jmethodID methodId;
    std::string ret;

    JNIEnv *env = GetJniEnv();
    if(env == NULL)
    {
        return ret;
    }

    cls = env->FindClass(HIVE_ANALYTICS_PLUGIN_CLASS);
    if (env->ExceptionCheck())
    {
        jthrowable exception = env->ExceptionOccurred();
        env->ExceptionDescribe();
        env->ExceptionClear();
        env->DeleteLocalRef(exception);
        return ret;
    }
    std::string signature = "(" + std::string(getJNISignature(arguments...)) + ")" + "Ljava/lang/String;";

    methodId = env->GetStaticMethodID(cls, methodName.c_str(), signature.c_str());
    if (env->ExceptionCheck())
    {
        jthrowable exception = env->ExceptionOccurred();
        env->ExceptionClear();
        env->DeleteLocalRef(exception);
        return ret;
    }
    LocalRefMapType localRefs;
    jstring jret = (jstring)env->CallStaticObjectMethod(cls, methodId, convert(localRefs, env, arguments)...);
    ret = getStringUTFCharsJNI(env, jret);
    env->DeleteLocalRef(cls);
    env->DeleteLocalRef(jret);
    deleteLocalRefs(env, localRefs);
    return ret;
}

void HIVEAnlayticsLog_AdLog(const char* eventType, const char* adUnitId, const char* uniqueSessionKey, const char* network, const char* additionalInfo, const char* rewardType, double rewardAmount) {
    HIVEAnalyticsLogPlugin_callJNI("AdLog", eventType, adUnitId, uniqueSessionKey, network, additionalInfo, rewardType, rewardAmount);
}

const char* HIVEAnalyticsLog_CreateUniqueSessionKey() {
    return HIVEAnalyticsLogPlugin_callJNI_backString("CreateUniqueSessionKey").c_str();
}
#endif
